-------------------------------------------
--- Author: Ketho (EU-Boulderfist)		---
--- License: Public Domain				---
--- Created: 2011.11.27					---
--- Version: 0.5 [2012.03.19]			---
-------------------------------------------
--- Curse			http://www.curse.com/addons/wow/guildleveltooltip
--- WoWInterface	http://www.wowinterface.com/downloads/info20527-GuildLevelTooltip.html
--- Request			http://www.curse.com/forums/games/world-of-warcraft/addons/148517-got-a-request

-- Thanks to Cowmonster/Isoloedlk (US-Khaz Modan) for a great amount of help with the inspect code

local VERSION = 0.5
local NAME = ...

local db

local f = CreateFrame("Frame")

	--------------
	--- Events ---
	--------------

function f:ADDON_LOADED(name)
	if name == NAME then
		GuildLevelTooltipDB = GuildLevelTooltipDB or {}
		GuildLevelTooltipDB.version = VERSION
		local realm = GetRealmName()
		GuildLevelTooltipDB[realm] = GuildLevelTooltipDB[realm] or {}
		db = GuildLevelTooltipDB[realm]
	end
end

local newUnit = "target"

-- Blizzard_InspectUI not loaded -> safe to NotifyInspect()
-- Blizzard_InspectUI loaded, hidden -> fix with InspectFrame.unit = "player", then NotifyInspect()
-- Blizzard_InspectUI loaded, shown -> don't NotifyInspect(), also not needed anyway

local function Notify(unit)
	if CanInspect(unit) and time() > (f.cd or 0) then
		-- NotifyInspect() is throttled by 6 calls for every 10 sec
		f.cd = time() + 2; newUnit = unit
		
		-- bit odd code, but favoring readability/simplicity
		if InspectFrame then
			if InspectFrame:IsShown() then
				return
			else
				InspectFrame.unit = "player"
				NotifyInspect(unit)
			end
		else
			NotifyInspect(unit)
		end
	end
end

function f:UPDATE_MOUSEOVER_UNIT()
	Notify("mouseover")
end

function f:PLAYER_TARGET_CHANGED()
	Notify("target")
end

function f:INSPECT_READY(guid)
	-- Blizzard_InspectUI is loaded and shown
	if InspectFrame and InspectFrame:IsShown() then
		newUnit = "target" -- unit is mostly "target" on manual inspect
	end
	
	-- INSPECT_READY passes the unit's guid instead of the unit id
	if UnitGUID(newUnit) == guid then
		local name = GetGuildInfo(newUnit)
		if not name then return end
		
		local _, level, xp, members = GetInspectGuildInfo(newUnit)
		if level == 0 then return end
		
		db[name] = db[name] or {}
		db[name][1] = level
		db[name][2] = members
		--db[name][3] = xp -- maybe useful for future versions
		--db[name][4] = time()
		--db[name][5] = date("%Y.%m.%d %H:%M")
	end
end

function f:OnEvent(event, ...)
	self[event](self, ...)
end

f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
f:RegisterEvent("PLAYER_TARGET_CHANGED")
f:RegisterEvent("INSPECT_READY")
f:SetScript("OnEvent", f.OnEvent)

	------------
	--- Hook ---
	------------

local function OnTooltipSetUnit()
	local unit = select(2, GameTooltip:GetUnit())
	if not unit then return end
	
	local guild = db[GetGuildInfo(unit)]
	if not guild then return end
	
	GameTooltip:AddLine(format(GUILD_LEVEL, guild[1]))
	GameTooltip:AddLine(format(INSPECT_GUILD_NUM_MEMBERS, guild[2]))
	GameTooltip:Show() -- update tooltip
end

GameTooltip:HookScript("OnTooltipSetUnit", OnTooltipSetUnit)
